@@ -1,14 +1,16 @@
 """Unit tests for zeroconf._services.info."""
 
+from __future__ import annotations
+
 import asyncio
 import logging
 import os
 import socket
 import threading
 import unittest
+from collections.abc import Iterable
 from ipaddress import ip_address
 from threading import Event
-from typing import Iterable, List, Optional
 from unittest.mock import patch
 
 import pytest
@@ -264,7 +266,7 @@ class TestServiceInfo(unittest.TestCase):
         send_event = Event()
         service_info_event = Event()
 
-        last_sent: Optional[r.DNSOutgoing] = None
+        last_sent: r.DNSOutgoing | None = None
 
         def send(out, addr=const._MDNS_ADDR, port=const._MDNS_PORT, v6_flow_scope=()):
             """Sends an outgoing packet."""
@@ -407,7 +409,7 @@ class TestServiceInfo(unittest.TestCase):
         send_event = Event()
         service_info_event = Event()
 
-        last_sent: Optional[r.DNSOutgoing] = None
+        last_sent: r.DNSOutgoing | None = None
 
         def send(out, addr=const._MDNS_ADDR, port=const._MDNS_PORT, v6_flow_scope=()):
             """Sends an outgoing packet."""
@@ -534,7 +536,7 @@ class TestServiceInfo(unittest.TestCase):
         send_event = Event()
         service_info_event = Event()
 
-        last_sent = None  # type: Optional[r.DNSOutgoing]
+        last_sent: r.DNSOutgoing | None = None
 
         def send(out, addr=const._MDNS_ADDR, port=const._MDNS_PORT, v6_flow_scope=()):
             """Sends an outgoing packet."""
@@ -666,7 +668,7 @@ class TestServiceInfo(unittest.TestCase):
 
 def test_multiple_addresses():
     type_ = "_http._tcp.local."
-    registration_name = "xxxyyy.%s" % type_
+    registration_name = f"xxxyyy.{type_}"
     desc = {"path": "/~paulsm/"}
     address_parsed = "10.0.1.2"
     address = socket.inet_aton(address_parsed)
@@ -830,7 +832,7 @@ def test_scoped_addresses_from_cache():
 async def test_multiple_a_addresses_newest_address_first():
     """Test that info.addresses returns the newest seen address first."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     desc = {"path": "/~paulsm/"}
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     cache = aiozc.zeroconf.cache
@@ -849,7 +851,7 @@ async def test_multiple_a_addresses_newest_address_first():
 @pytest.mark.asyncio
 async def test_invalid_a_addresses(caplog):
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     desc = {"path": "/~paulsm/"}
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     cache = aiozc.zeroconf.cache
@@ -879,7 +881,7 @@ def test_filter_address_by_type_from_service_info():
     ipv6 = socket.inet_pton(socket.AF_INET6, "2001:db8::1")
     info = ServiceInfo(type_, registration_name, 80, 0, 0, desc, "ash-2.local.", addresses=[ipv4, ipv6])
 
-    def dns_addresses_to_addresses(dns_address: List[DNSAddress]) -> List[bytes]:
+    def dns_addresses_to_addresses(dns_address: list[DNSAddress]) -> list[bytes]:
         return [address.address for address in dns_address]
 
     assert dns_addresses_to_addresses(info.dns_addresses()) == [ipv4, ipv6]
@@ -1082,7 +1084,7 @@ async def test_we_try_four_times_with_random_delay():
 async def test_release_wait_when_new_recorded_added():
     """Test that async_request returns as soon as new matching records are added to the cache."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     desc = {"path": "/~paulsm/"}
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
@@ -1147,7 +1149,7 @@ async def test_release_wait_when_new_recorded_added():
 async def test_port_changes_are_seen():
     """Test that port changes are seen by async_request."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     desc = {"path": "/~paulsm/"}
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
@@ -1230,7 +1232,7 @@ async def test_port_changes_are_seen():
 async def test_port_changes_are_seen_with_directed_request():
     """Test that port changes are seen by async_request with a directed request."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     desc = {"path": "/~paulsm/"}
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
@@ -1313,7 +1315,7 @@ async def test_port_changes_are_seen_with_directed_request():
 async def test_ipv4_changes_are_seen():
     """Test that ipv4 changes are seen by async_request."""
     type_ = "_http._tcp.local."
-    registration_name = "multiaipv4rec.%s" % type_
+    registration_name = f"multiaipv4rec.{type_}"
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
 
@@ -1401,7 +1403,7 @@ async def test_ipv4_changes_are_seen():
 async def test_ipv6_changes_are_seen():
     """Test that ipv6 changes are seen by async_request."""
     type_ = "_http._tcp.local."
-    registration_name = "multiaipv6rec.%s" % type_
+    registration_name = f"multiaipv6rec.{type_}"
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
 
@@ -1496,7 +1498,7 @@ async def test_ipv6_changes_are_seen():
 async def test_bad_ip_addresses_ignored_in_cache():
     """Test that bad ip address in the cache are ignored async_request."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
 
@@ -1550,7 +1552,7 @@ async def test_bad_ip_addresses_ignored_in_cache():
 async def test_service_name_change_as_seen_has_ip_in_cache():
     """Test that service name changes are seen by async_request when the ip is in the cache."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
 
@@ -1632,7 +1634,7 @@ async def test_service_name_change_as_seen_has_ip_in_cache():
 async def test_service_name_change_as_seen_ip_not_in_cache():
     """Test that service name changes are seen by async_request when the ip is not in the cache."""
     type_ = "_http._tcp.local."
-    registration_name = "multiarec.%s" % type_
+    registration_name = f"multiarec.{type_}"
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahost.local."
 
@@ -1715,7 +1717,7 @@ async def test_service_name_change_as_seen_ip_not_in_cache():
 async def test_release_wait_when_new_recorded_added_concurrency():
     """Test that concurrent async_request returns as soon as new matching records are added to the cache."""
     type_ = "_http._tcp.local."
-    registration_name = "multiareccon.%s" % type_
+    registration_name = f"multiareccon.{type_}"
     desc = {"path": "/~paulsm/"}
     aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
     host = "multahostcon.local."
@@ -1786,7 +1788,7 @@ async def test_release_wait_when_new_recorded_added_concurrency():
 async def test_service_info_nsec_records():
     """Test we can generate nsec records from ServiceInfo."""
     type_ = "_http._tcp.local."
-    registration_name = "multiareccon.%s" % type_
+    registration_name = f"multiareccon.{type_}"
     desc = {"path": "/~paulsm/"}
     host = "multahostcon.local."
     info = ServiceInfo(type_, registration_name, 80, 0, 0, desc, host)
@@ -1795,3 +1797,77 @@ async def test_service_info_nsec_records():
     assert nsec_record.type == const._TYPE_NSEC
     assert nsec_record.ttl == 50
     assert nsec_record.rdtypes == [const._TYPE_A, const._TYPE_AAAA]
+
+
+@pytest.mark.asyncio
+async def test_address_resolver():
+    """Test that the address resolver works."""
+    aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
+    await aiozc.zeroconf.async_wait_for_start()
+    resolver = r.AddressResolver("address_resolver_test.local.")
+    resolve_task = asyncio.create_task(resolver.async_request(aiozc.zeroconf, 3000))
+    outgoing = r.DNSOutgoing(const._FLAGS_QR_RESPONSE)
+    outgoing.add_answer_at_time(
+        r.DNSAddress(
+            "address_resolver_test.local.",
+            const._TYPE_A,
+            const._CLASS_IN,
+            10000,
+            b"\x7f\x00\x00\x01",
+        ),
+        0,
+    )
+
+    aiozc.zeroconf.async_send(outgoing)
+    assert await resolve_task
+    assert resolver.addresses == [b"\x7f\x00\x00\x01"]
+
+
+@pytest.mark.asyncio
+async def test_address_resolver_ipv4():
+    """Test that the IPv4 address resolver works."""
+    aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
+    await aiozc.zeroconf.async_wait_for_start()
+    resolver = r.AddressResolverIPv4("address_resolver_test_ipv4.local.")
+    resolve_task = asyncio.create_task(resolver.async_request(aiozc.zeroconf, 3000))
+    outgoing = r.DNSOutgoing(const._FLAGS_QR_RESPONSE)
+    outgoing.add_answer_at_time(
+        r.DNSAddress(
+            "address_resolver_test_ipv4.local.",
+            const._TYPE_A,
+            const._CLASS_IN,
+            10000,
+            b"\x7f\x00\x00\x01",
+        ),
+        0,
+    )
+
+    aiozc.zeroconf.async_send(outgoing)
+    assert await resolve_task
+    assert resolver.addresses == [b"\x7f\x00\x00\x01"]
+
+
+@pytest.mark.asyncio
+@unittest.skipIf(not has_working_ipv6(), "Requires IPv6")
+@unittest.skipIf(os.environ.get("SKIP_IPV6"), "IPv6 tests disabled")
+async def test_address_resolver_ipv6():
+    """Test that the IPv6 address resolver works."""
+    aiozc = AsyncZeroconf(interfaces=["127.0.0.1"])
+    await aiozc.zeroconf.async_wait_for_start()
+    resolver = r.AddressResolverIPv6("address_resolver_test_ipv6.local.")
+    resolve_task = asyncio.create_task(resolver.async_request(aiozc.zeroconf, 3000))
+    outgoing = r.DNSOutgoing(const._FLAGS_QR_RESPONSE)
+    outgoing.add_answer_at_time(
+        r.DNSAddress(
+            "address_resolver_test_ipv6.local.",
+            const._TYPE_AAAA,
+            const._CLASS_IN,
+            10000,
+            socket.inet_pton(socket.AF_INET6, "fe80::52e:c2f2:bc5f:e9c6"),
+        ),
+        0,
+    )
+
+    aiozc.zeroconf.async_send(outgoing)
+    assert await resolve_task
+    assert resolver.ip_addresses_by_version(IPVersion.All) == [ip_address("fe80::52e:c2f2:bc5f:e9c6")]
